import requests
import os

from cloudhealth import exceptions
from .assets import AssetsClient
from .reports import ReportingClient
from .sso import SsoClient

class Client():
    def __init__(self,
                 endpoint,
                 api_key):
        self.endpoint = endpoint
        self.api_key = api_key
        self.headers = {
            'Authorization': f'Bearer {self.api_key}',
            'Accept': 'application/json'
        }

    def query(self,
            uri,
            data=None,
            params=None,
            headers={},
            method='GET'):

        url = f'{self.endpoint}{uri}'

        if len(url) >= 4000:
            raise exceptions.CloudHealthError(
                'The maximum permissible length of a URI string is 4000 characters.'
            )

        headers.update(self.headers)
        
        response = requests.request(
            method,
            url,
            data=data,
            params=params,
            headers=headers)

        if response.status_code not in [200, 201, 204]:
            error = response.json().get('error')
            raise exceptions.CloudHealthError(f'{response.status_code} - {error}')

        return response.json()


class CloudHealth():
    def __init__(self, endpoint='https://chapi.cloudhealthtech.com/', api_key=None):
        if not api_key:
            try:
                api_key = os.environ['CLOUDHEALTH_API_KEY']
            except KeyError:
                raise exceptions.CloudHealthError('API_KEY not set')

        self._client = Client(endpoint, api_key)
        self.assets = AssetsClient(self._client)
        self.reports = ReportingClient(self._client)
        self.sso = SsoClient(self._client)



from . import exceptions

class ReportingClient():

    def __init__(self, client):
        self.client = client
        self.uri = 'olap_reports'

    def list(self, org_id=None):
        uri = f'{self.uri}'
        params = [('org_id', org_id)]

        reports = self.client.query(uri, method='GET', params=params)

        return reports


    def list_type(self, report_type, org_id=None):
        uri = f'{self.uri}/{report_type}'
        params = [('org_id', org_id)]

        reports = self.client.query(uri, method='GET', params=params)

        return reports


    def get_data(self, report_type, report_id, dimensions=[], 
            measures=[], interval='', filters=[], org_id='', collapse_null_arrays=True,
            no_cache=True):
        uri = f'{self.uri}/{report_type}/{report_id}'

        params = [
            *[('dimensions[]', v) for v in dimensions],
            *[('measures[]', v) for v in measures],
            *[('filters[]', v) for v in filters],
            *[('interval', interval),
              ('org_id', org_id),
              ('collapse_null_arrays', 1 if collapse_null_arrays else 0),
              ("NO_CACHE", 1 if no_cache else 0)]
        ]

        data = self.client.query(uri, method='GET', params=params)

        return data

    def get_dimensions(self, report_type, report_id, org_id=''):
        uri = f'{self.uri}/{report_type}/{report_id}/new'
        params = [('org_id', org_id)]

        dimensions = self.client.query(uri, method='GET', params=params)

        return dimensions



from . import exceptions

class SsoClient():

    def __init__(self, client):
        self.client = client


    def configure_sso(self, sso_provider, domains, endpoint, certificate, 
					  client_api_id='', default_organization_id='', 
                      sso_ignore_idp_organization=False,
					  azure_ad_roles_protocol=True):
        uri = '/v1/sso/configure'
        params = [('client_api_id', client_api_id)]
        data = {
			'sso_provider': sso_provider,
			'domains': domains,
			'default_organization_id': default_organization_id,
			'endpoint': endpoint,
			'certificate': certificate,
			'sso_ignore_idp_organization': sso_ignore_idp_organization,
			'azure_ad_roles_protocol': azure_ad_roles_protocol
        }

        response = self.client.query(
            uri, method='PUT', data=json.dumps(data), params=params
        )

        return response


    def get_configuration(self, client_api_id=''):
        uri = '/v1/sso/configuration'
        params = [('client_api_id', client_api_id)]

        configuration = self.client.query(uri, method='GET', params=params)

        return configuration


    def delete_configuration(self, client_api_id=''):
        uri = '/v1/sso/unconfigure'
        params = [('client_api_id', client_api_id)]

        response = self.client.query(uri, method='DELETE', params=params)

        return response


    def get_pending_domains(self, client_api_id=''):
        uri = '/v1/sso/pending_domain_claims'
        params = [('client_api_id', client_api_id)]

        configuration = self.client.query(uri, method='GET', params=params)

        return configuration


    def validate_pending_domain(self, pending_domain_name, client_api_id=''):
        uri = '/v1/sso/validate_pending_domain_claim'
        params = [('client_api_id', client_api_id)]
        data = {"pending_domain_name": pending_domain_name}

        response = self.client.query(
			uri, method='PUT', data=json.dumps(data), params=params
		)

        return response



import json
import calendar

from datetime import date, timedelta, datetime


def _get_yesterdays_date():
    current = date.today()
    current_time = datetime.utcnow()
    if current.day == 1:
        if current_time.hour <= 5:
            return date(current.year, current.month-1, (calendar.monthrange(current.year, current.month-1)[2])).strftime('%Y-%m-%d')
        else:
            return date(current.year, current.month-1, (calendar.monthrange(current.year, current.month-1)[1])).strftime('%Y-%m-%d')
    else:
        if current_time.hour <= 5:
            return date(current.year, current.month, current.day-2).strftime('%Y-%m-%d')
        else:
            return date(current.year, current.month, current.day-1).strftime('%Y-%m-%d')

def _get_last_month():
    current = date.today()
    if current.day == 31:
        return date(current.year, current.month-1, (calendar.monthrange(current.year, current.month-1)[1])).strftime('%Y-%m')
    else:
        return date(current.year, current.month-1, current.day).strftime('%Y-%m')

def _format_json(dictionary):
    return json.dumps(dictionary, indent=4, sort_keys=True)


from . import exceptions

class AssetsClient():

    def __init__(self, client):
        self.client = client

    def list(self, org_id=None):
        uri = '/api'
        params = [('org_id', org_id)]

        assets = self.client.query(uri, method='GET', params=params)

        return assets


    def get_attributes(self, asset, org_id=None):
        uri = f'api/{asset}'
        params = [('org_id', org_id)]

        attributes = self.client.query(uri, method='GET', params=params)

        return attributes


    def search(self, name=None, query=None, include=None, org_id=None,
        api_version=2, fields=None, page=1, per_page=50):
        uri = '/api/search'

        params = [
            ("name", name),
            ("query", query),
            ("include", include),
            ("org_id", org_id),
            ("api_version", api_version),
            ("fields", fields),
            ("page", page),
            ("per_page", per_page)
        ]

        assets = self.client.query(uri, method='GET', params=params)
        return assets



from setuptools import setup

setup(
    name='cloudhealth-client',
    version='0.1',
    author='Alberto Narro',
    author_email='alberto.narro@hotmail.com',
    url = 'https://github.com/albertonarro/cloudhealth-client',
    keywords = ['CLOUDHEALTH', 'CLIENT', 'CLI'],
    packages=['cloudhealth'],
    license='LICENSE',
    description='A REST Client for Cloudhealth',
    install_requires=[
        'requests==2.28.2',
    ]
)
